*DECK FLUADN
      SUBROUTINE FLUADN(IPFLUX,IPMACR,IPRINT,NGROUP,NBMIX ,NREGIO,
     >                  NUNKNO,NIFISS,FUNKNO,MATCOD,VOLUME,KEYFLX,
     >                  IOUTER,
     >                  SIGF0 ,FFDIR ,QFDIR ,FLUA  )
C
C---------------------------  FLUADN  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C      NAME     : FLUADN
C      USE      : GENERALIZED ADJOINT RENORMALISATION
C      MODIFIED : 2000/01/18 (T. COURAU)
C
C   2- ROUTINE PARAMETERS:
C      IPFLUX  : POINTER TO THE SOLUTION                   I
C      IPMACR  : POINTER TO THE MACROLIB                   I
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES          I
C                = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
C      NGROUP  : NUMBER OF GROUPS.                         I
C      NBMIX   : NUMBER OF MIXTURES.                       I
C      NREGIO  : NUMBER OF REGIONS.                        I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.          I
C      NIFISS  : NUMBER OF FISSILE ISOTOPES.               I
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.                R(NUNKNO,
C                                                            NGROUP)
C      MATCOD  : MATERIAL CODE IN REGIONS.                 I(NREGIO)
C      VOLUME  : VOLUME OF REGIONS. DIMENSION              R(NREGIO)
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.          I(NREGIO)
C      IOUTER  : OUTER ITERATION NUMBER                    I
C      SIGF0   : FISSION CROSS SECTION                     R(NBMIX,
C                                                            NIFISS)
C      FFDIR   : FISSION SOURCE VECTOR                     R(NUNKNO,
C                                                            NGROUP)
C      QFDIR   : FISSION CROSS SECTION                     R(NREGIO,
C                                                            NIFISS)
C      FLUA    : FISSION SOURCE VECTOR                     R(NUNKNO,
C                                                            NGROUP)
C---------------------------   FLUADN  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='FLUADN')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPFLUX,IPMACR,IPRINT,NGROUP,NBMIX,NREGIO,
     >            NUNKNO,NIFISS,IOUTER
      INTEGER     MATCOD(NREGIO),KEYFLX(NREGIO)
      REAL        FUNKNO(NUNKNO,NGROUP),VOLUME(NREGIO),
     >            SIGF0(NBMIX,NIFISS),FFDIR(NUNKNO,NGROUP),
     >            QFDIR(NREGIO,NIFISS),FLUA(NUNKNO,NGROUP)
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CDIRFL*12,CGRPNM*12,CNUF*12,CCHI*12
      INTEGER     IGR,IREG,IFIS,IBM,IFPOS
      DOUBLE PRECISION   B1,B2
C----
C  INITIALISATION OF QFDIR AND FFDIR FOR THE FIRST OUTER ITERATION
C----
      CNUF='NUSIGF      '
      CCHI='CHI         '
      IF(IOUTER .EQ. 1) THEN
        CALL XDRSET(QFDIR,NREGIO*NIFISS,0.0)
        CDIRFL='FLUXDIRECT  '
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
        DO 100 IGR=1,NGROUP
          WRITE(CGRPNM,'(4HFLUX,I3)') IGR
          CALL LCMGET(IPFLUX,CGRPNM,FFDIR(1,IGR))
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
          CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
          CALL LCMGET(IPMACR,CNUF,SIGF0)
C----
C  TOTAL FISSION SOURCE PER ISOTOPE COMPUTATION
C----
          DO 101 IREG=1,NREGIO
            IBM=MATCOD(IREG)
            IFPOS=KEYFLX(IREG)
            IF(IBM .GT. 0) THEN
              DO 102 IFIS=1,NIFISS
                QFDIR(IREG,IFIS)=QFDIR(IREG,IFIS)
     >            +VOLUME(IREG)*FFDIR(IFPOS,IGR)*SIGF0(IBM,IFIS)
 102          CONTINUE
            ENDIF
 101      CONTINUE
          CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 100    CONTINUE
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
        CALL XDRSET(FFDIR,NUNKNO*NGROUP,0.0)
        DO 110 IGR=1,NGROUP
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
          CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
          CALL LCMGET(IPMACR,CCHI,SIGF0)
C----
C  FISSION SOURCE TERM COMPUTATION
C----
          DO 111 IREG=1,NREGIO
            IBM=MATCOD(IREG)
            IFPOS=KEYFLX(IREG)
            IF(IBM .GT. 0) THEN
              DO 112 IFIS=1,NIFISS
                FFDIR(IFPOS,IGR)=FFDIR(IFPOS,IGR)
     >            +QFDIR(IREG,IFIS)*SIGF0(IBM,IFIS)
 112          CONTINUE
            ENDIF
 111      CONTINUE
          CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 110    CONTINUE
      ENDIF
      B1=0.0D0
      B2=0.0D0
C-----
C  SCALAR PRODUCTS COMPUTATION
C  B1 : ADJOINT FLUX
C  B2 : GENERALIZED ADJOINT FLUX
C-----
      CDIRFL='PSEUDOADJ   '
      CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
      DO 120 IGR=1,NGROUP
        WRITE(CGRPNM,'(4HFLUX,I3)') IGR
        CALL LCMGET(IPFLUX,CGRPNM,FLUA(1,IGR))
        DO 121 IREG=1,NREGIO
          IFPOS=KEYFLX(IREG)
          B1=B1+DBLE(FUNKNO(IFPOS,IGR)*FFDIR(IFPOS,IGR))
          B2=B2+DBLE(FLUA(IFPOS,IGR)*FFDIR(IFPOS,IGR))
 121    CONTINUE
 120  CONTINUE
      CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
      DO 130 IGR=1,NGROUP
C----
C  RENORMALIZED FLUX STORED IN FUNKNO
C----
        DO 131 IREG=1,NREGIO
          IFPOS=KEYFLX(IREG)
          FUNKNO(IFPOS,IGR)=FUNKNO(IFPOS,IGR)
     >       -REAL(B1/B2)*FLUA(IFPOS,IGR)
 131     CONTINUE
 130    CONTINUE
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6000) NAMSBR,-B1/B2
      ENDIF
      RETURN
C----
C  FORMAT
C----
 6000 FORMAT(1X,A6,1X,'NORMALIZATION FACTOR =',1P,E15.7/)
      END
