*DECK FLUQFE
      SUBROUTINE FLUQFE(IPFLUX,IPSYS ,IPMACR,IPRINT,LEAKSW,ITYPEC,
     >                  ISADJ ,ILEAK ,IOUTER,NGROUP,NBMIX ,NREGIO,
     >                  NUNKNO,NIFISS,MATCOD,VOLUME,KEYFLX,EPSEIG,
     >                  EPSFLX,COPTIO,REFKEF,B2    ,EIGENK,LCONV ,
     >                  CFIXSR,FUNKNO,SUNKNO,
     >                  DB2   ,SIGT0 ,SIGS0 ,XSCAT ,IJJ   ,NJJ   ,
     >                  IPOS  ,SIGTH ,SIGF0 ,QTOTL ,RELFOL,LOSCIL)
C
C---------------------------  FLUQFE  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : FLUQFE
C       USE      : SOURCE EVALUATION (FISSION+FIXED), FLUX NORMALIZATION
C                  EIGENVALUE EVALUATION, B-N LEAKAGE CALCULATIONS AND
C                  FLUX CONVERGENCE CHECKING
C       MODIFIED : MODIFY DEFINITION OF ITYPEC
C                  2000/01/27  (G. MARLEAU)
C                  2000/01/20  (T. COURAU, G. MARLEAU)
c                  1994/10/27  (A. HEBERT)
C                  1994/07/21  (I. PETROVIC, G. MARLEAU)
C                  1991/03/15  (G. MARLEAU)
C                  1991/01/28  (A. HEBERT)
C                  1986/05/14  (G.MARLEAU)
C
C   2- ROUTINE PARAMETERS:
C    INPUT:
C      IPFLUX  : POINTER TO THE SOLUTION              I
C      IPSYS   : POINTER TO THE PIJ                   I
C      IPMACR  : POINTER TO THE MACROLIB              I
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES:    I
C                = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
C      LEAKSW  : LEAKAGE SWITCH.                      L
C      ITYPEC  : TYPE OF FLUX EVALUATION              I
C                = 0  SKIP THE FLUX CALCULATION
C                = 1  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE
C                = 2  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     FIXED DB2 BUCKLING
C                = 3  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     DB2 BUCKLING EVALUATION
C                = 4  DIRECT PROBLEM FISSION SOURCES/
C                     DB2 BUCKLING CONVERGENCE
C                = 5  DIRECT PROBLEM DB2 SOURCES
C                = 6  DIRECT PROBLEM FIXED SOURCES
C      ISADJ   : ADJOINT CALCULATION AND SAVE OPTION  I
C                = 0 ONLY DIRECT FLUX
C                = 1 DIRECT AND PSEUDO ADJOINT FLUX
C                = 2 DIRECT, PSEUDO-ADJOINT AND ADJOINT
C                = 3 DIRECT, PSEUDO ADJOINT AND
C                    GENERALIZED PSEUDO-ADJOINT
C                = 4 DIRECT, PSEUDO ADJOINT, ADJOINT,
C                    GENERALIZED PSEUDO-ADJOINT AND
C                    GENERALIZED ADJOINT
C      ILEAK   : METHOD USED TO INCLUDE DB2 EFFECT    I
C                = 0 NO DB2 EFFECT
C                = 1 THE SCATTERING MODIFIED CP MATRIX
C                    IS MULTIPLIED BY A PNL FACTOR
C                = 2 THE REDUCED CP MATRIX
C                    MULTIPLIED BY PNL FACTOR
C                = 3 SIGS0-DB2 APPROXIMATION
C                = 4 ALBEDO APPROXIMATION
C                = 5,6,7,8,9,10 HETEROGENEOUS WITH PIJK
C                  AND FIXED B->5 OR
C                  SEARCH X->6 Y->7 Z->8 R->9 ALL->10
C      IOUTER  : OUTER ITERATION NUMBER               I
C      NGROUP  : NUMBER OF GROUPS.                    I
C      NBMIX   : NUMBER OF MIXTURES.                  I
C      NREGIO  : NUMBER OF REGIONS.                   I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.     I
C      NIFISS  : NUMBER OF FISSILE ISOTOPES.          I
C      MATCOD  : MATERIAL CODE IN REGIONS.            I(NREGIO)
C      VOLUME  : VOLUME OF REGIONS.                   R(NREGIO)
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.     I(NREGIO)
C      EPSEIG  : EIGENVALUE TOLERANCE                 R
C      EPSFLX  : FLUX AND CURRENT TOLERANCE           R
C      COPTIO  : CAN BE 'LKRD' (IMPOSED LEAKAGE),     C*4
C                'B0' (B-0), 'P0' (P-0), 'B1' (B-1),
C                'P1' (P-1) OR
C                'B0TR' (B-0  TRANSPORT CORRECTION).
C      REFKEF  : TARGET K-EFFECTIVE FOR TYPE B OR L   D
C                CRITICAL K FOR GENERALIZED ADJOINT
C      CFIXSR  : NAME OF GENERALIZED ADJOINT SOURCE   C*12
C    INPUT/OUTPUT
C      B2      : INITIAL ESTIMATE OF THE BUCKLING     R(4)
C                OR IMPOSED BUCKLING AND
C                CONVERGED BUCKLING IF
C                ITYPEC =  3,  4,
C      EIGENK  : INITIAL K-EFFECTIVE AND              R
C                CONVERGED K-EFFECTIVE IF
C                ITYPEC =  1, 2, 3, 4.
C    OUTPUT
C      LCONV   : =.TRUE. IF CONVERGENCE IS REACHED.   L
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.           R(NUNKNO,NGROUP)
C      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.        R(NUNKNO,NGROUP)
C      DB2     : LEAKAGE PARAMETERS                   R(NGROUP)
C                FOR PIJ  -> DB2(G)=D(G)*B**2
C                FOR PIJK -> DB2(G)=LAMBDA(B/SIGMA(G))
C      RELFOL  : RELATIVE CONVERGENCE OF THE FLUX     R
C      LOSCIL  : FLAG ACTIVATED IN CASE OF 
C                OSCILLATIONS                         L
C    SCRATCH STORAGE:
C      SIGT0   : TOTAL CROSS SECTION VECTOR           R(0:NBMIX)
C                SIGT0(0)=0.0 IS FOR VOID
C      SIGS0   : WITHIN-GROUP SCATTERING              R(0:NBMIX)
C                SIGS0(0)=0.0 IS FOR VOID
C      XSCAT   : SCATTERING VECTOR                    R(0:NBMIX*NGROUP)
C                XSCAT(0) = 0.0
C      NJJ     : NUMBER OF SCATTERING GROUP           I(0:NBMIX)
C                NJJ(0) = -NGROUP
C      IJJ     : LAST SCATTERING GROUP                I(0:NBMIX)
C                IJJ(0) = 0
C      IPOS    : POSITION SELF SCATTERING IN XSCAT    I(0:NBMIX)
C                IPOS(0)= NGROUP+1
C      SIGTH   : TOTAL HOMO CROSS SECTION VECTOR      R(NGROUP)
C      SIGF0   : NUSIGF CROSS SECTION VECTOR          R(NBMIX,NIFISS)
C      QTOTL   : TOTAL FISSION SOURCE                 R(NREGIO,NIFISS)
C
C---------------------------   FLUQFE  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='FLUQFE')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPFLUX,IPSYS,IPMACR,IPRINT,ITYPEC,ISADJ,ILEAK,
     >            IOUTER,NGROUP,NBMIX,NREGIO,NUNKNO,NIFISS,MAXREG,GRMAX
      CHARACTER   COPTIO*4,CFIXSR*12
      INTEGER     MATCOD(NREGIO),KEYFLX(NREGIO),
     >            IJJ(0:NBMIX),NJJ(0:NBMIX),IPOS(0:NBMIX)
      LOGICAL     LEAKSW,LCONV,LOSCIL
      REAL        VOLUME(NREGIO),EPSEIG,EPSFLX,B2(4),EIGENK,
     >            FUNKNO(NUNKNO,NGROUP),SUNKNO(NUNKNO,NGROUP),
     >            DB2(NGROUP),SIGT0(0:NBMIX),SIGS0(0:NBMIX),
     >            XSCAT(0:NBMIX*NGROUP),SIGTH(0:NBMIX),
     >            SIGF0(NBMIX,NIFISS),QTOTL(NREGIO,NIFISS),RELFOL
      DOUBLE PRECISION REFKEF
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IDIFFU,IPHI,IPPP,IDELFL
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CGRPNM*12,CAN*3,CAT*3,CNUF*12,CCHI*12
      LOGICAL     LCFLX,LCCOU,LCEIG
      INTEGER     JJ,IGENK,IBUCK,IGR,IREG,ILCMLN,ILCMTY,INORM,IHETL,
     >            IBM,IFLXL,IFIS,IG2,IXSS,KCOU,IPOIND,IPOINF,IDIR,
     >            IMIX
      REAL        B2OLD,B2NEW,RELEIG,RELCOU,RELFLX,TDPHI,TMPHI,
     >            TDCOU,TMCOU,FIMOY
      DOUBLE PRECISION OLDBIL,CUREIN,BIL1,BIL2,FACTNO,EIGNEW,
     >            ALAM1,DSIGS

C----
C  DATA
C----
      CHARACTER   HTYPE(0:3)*4
      SAVE        HTYPE
      DATA       (HTYPE(JJ),JJ=0,3)
     >           /'S','K','B','L'/
C----
C  INITIALIZE LOCAL CALCULATION OPTION
C  IGENK:
C    0 -> FIXED SOURCE OR GENERALIZED ADJOINT SOURCES
C    1 -> K EIGENVALUE
C    2 -> B-HOMO EIGENVALUE
C    3 -> B-HETE EIGENVALUE
C  IBUCK:
C    0 -> NO LEAKAGE
C    1 -> FIXED LEAKAGE
C    2 -> COMPUTED LEAKAGE
C----

      IGENK=1
      IBUCK=0
      CAN='D 0'
      CAT='A 0'
      CNUF='NUSIGF      '
      CCHI='CHI         '
      IF(ISADJ .GE. 1) THEN
        CAN='A 0'
        CAT='D 0'
        CNUF='CHI         '
        CCHI='NUSIGF      '
        IF(ISADJ .GE. 3 .OR. ITYPEC .GE. 6) THEN 
          IGENK=0
        ENDIF        
        IF(ITYPEC .GE. 2 .AND. ITYPEC .LE. 5) THEN
          IBUCK=1
        ENDIF
      ELSE
        IF(ITYPEC .GE. 6) THEN
          IGENK=0
        ELSE IF(ITYPEC .EQ. 5) THEN
          IGENK=3
          IBUCK=2
        ELSE IF(ITYPEC .EQ. 4) THEN
          IGENK=2
          IBUCK=2
        ELSE IF(ITYPEC .EQ. 3) THEN
          IBUCK=2
        ELSE IF(ITYPEC .EQ. 2) THEN
          IBUCK=1
        ENDIF
      ENDIF
C----
C  INITIALIZE THE FLUX IF IOUTER=0
C----
      LCFLX=.TRUE.
      LCCOU=.TRUE.
      OLDBIL=0.0D0
      IF(IOUTER .EQ. 0) THEN
        CALL FLUGFL(IPFLUX,NGROUP,NUNKNO,FUNKNO)
      ELSE
        IF(IGENK .GE. 1 ) THEN
          DO 11 IGR=1,NGROUP
             DO 12 IREG=1,NREGIO
              OLDBIL=OLDBIL
     >              +DBLE(VOLUME(IREG)*SUNKNO(KEYFLX(IREG),IGR))
 12         CONTINUE
 11       CONTINUE
        ENDIF
      ENDIF
C----
C  SET-UP THE SOURCES FOR THE NEXT MULTIREGION FLUX CALCULATION.
C----
      CALL XDRSET(QTOTL,NREGIO*MAX(NIFISS,1),0.0)
      CALL XDRSET(SUNKNO,NUNKNO*NGROUP,0.0)
      IF(IBUCK .GE. 1) THEN
        IF(ILEAK .GE. 5) THEN
          B2OLD=B2(1)+B2(2)+B2(3)
          INORM=-1
          IHETL=ILEAK-5
        ELSE
          B2OLD=B2(4)
          INORM=2
          IHETL=0
        ENDIF
      ELSE
        B2OLD=0.0
      ENDIF
      CUREIN=0.0D0
      IF(IGENK .GE. 1) THEN
        IF(IBUCK .EQ. 0) THEN
C----
C  NO B-N LEAKAGE CALCULATION REQUIRED
C----
          IF(B2(4) .NE. 0.0) B2(4)=0
          CALL XDRSET(DB2,NGROUP,0.0)
        ELSE IF(IBUCK .GT. 1) THEN
          CALL SETARA(RBASE,NGROUP,IDIFFU)
          CALL XDRSET(RBASE(IDIFFU),NGROUP,0.0)
C----
C  PERFORM THE B-N LEAKAGE CALCULATION.
C----
          IF(LEAKSW) CALL XABORT(NAMSBR//
     >    ': THE ALBEDO SHOULD BE EQUAL TO 1.')
          CALL SETARA(RBASE,2*NGROUP,IPHI)
          IF(IOUTER .EQ. 0) THEN
            IF(COPTIO .EQ. 'LKRD') THEN
              CALL LCMGET(IPFLUX,'DIFFB1HOM',RBASE(IDIFFU))
            ELSE
              CALL B1HOM(IPMACR,NUNKNO,COPTIO,'DIFF',NGROUP,NREGIO,
     >                   NBMIX,NIFISS,VOLUME,MATCOD,KEYFLX,FUNKNO,
     >                   REFKEF,0,RBASE(IDIFFU),ALAM1,SIGTH,
     >                   RBASE(IPHI),XSCAT,IJJ,NJJ,IPOS,INORM,B2)
              CALL LCMPUT(IPFLUX,'DIFFB1HOM',NGROUP,2,RBASE(IDIFFU))
              CALL LCMPUT(IPFLUX,'B2  B1HOM',1,2,B2(4))
            ENDIF
          ELSE
            CALL LCMGET(IPFLUX,'DIFFB1HOM',RBASE(IDIFFU))
            CALL B1HOM(IPMACR,NUNKNO,COPTIO,HTYPE(IGENK),NGROUP,
     >                 NREGIO,NBMIX,NIFISS,VOLUME,MATCOD,KEYFLX,
     >                 FUNKNO,REFKEF,IPRINT,RBASE(IDIFFU),ALAM1,
     >                 SIGTH,RBASE(IPHI),XSCAT,IJJ,NJJ,IPOS,
     >                 INORM,B2)
            CALL LCMPUT(IPFLUX,'DIFFB1HOM',NGROUP,2,RBASE(IDIFFU))
            CALL LCMPUT(IPFLUX,'B2  B1HOM',1,2,B2(4))
          ENDIF
          CALL RLSARA(RBASE(IPHI))
          IF(ILEAK .LT. 5) THEN
            B2NEW=B2(4)
            DO 210 IGR=1,NGROUP
              DB2(IGR)=B2(4)*RBASE(IDIFFU+IGR-1)
 210        CONTINUE
          ENDIF
          CALL RLSARA(RBASE(IDIFFU))
        ENDIF
C----
C  COMPUTE FISSION SOURCE AND EVALUATE NEUTRON BALANCE
C  BIL1 = TOTAL NEUTRON PRODUCTION IN THIS ITERATION
C----
        BIL1=0.0D0
        IF(NIFISS .GT. 0) THEN
          DO 20 IGR=1,NGROUP
            WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
            CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
            CALL LCMGET(IPMACR,CNUF,SIGF0)
            DO 21 IREG=1,NREGIO
              IBM=MATCOD(IREG)
              IF(IBM .GT. 0) THEN
                IFLXL=KEYFLX(IREG)
                DO 22 IFIS=1,NIFISS
                  QTOTL(IREG,IFIS)=QTOTL(IREG,IFIS)
     >               +FUNKNO(IFLXL,IGR)*SIGF0(IBM,IFIS)
 22             CONTINUE
              ENDIF
 21         CONTINUE
            CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 20       CONTINUE
          DO 30 IGR=1,NGROUP
            WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
            CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
            CALL LCMGET(IPMACR,CCHI,SIGF0)
            DO 31 IREG=1,NREGIO
              IBM=MATCOD(IREG)
              IF(IBM .GT. 0) THEN
                DO 32 IFIS=1,NIFISS
                  BIL1=BIL1+DBLE(VOLUME(IREG)*QTOTL(IREG,IFIS)*
     >                      SIGF0(IBM,IFIS))
 32             CONTINUE
              ENDIF
 31         CONTINUE
            CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 30       CONTINUE
        ENDIF
C----
C  BIL2 = TOTAL NEUTRON LOSS IN THIS ITERATION
C----
        BIL2=0.0D0
        IF(IBUCK .GE. 1) THEN
          IF(ILEAK .LT. 5) THEN
            DO 220 IGR=1,NGROUP
              DO 221 IREG=1,NREGIO
                BIL2=BIL2+DBLE(FUNKNO(KEYFLX(IREG),IGR)*DB2(IGR)
     >                   *VOLUME(IREG))
 221          CONTINUE
 220        CONTINUE
          ELSE
C----
C  IHETL : TYPE OF HETEROGENEOUS BUCKLING CALCULATION
C          = 0 -> ALL-DIRECTION IMPOSED
C          = 1 -> X-DIRECTION SEARCH
C          = 2 -> Y-DIRECTION SEARCH
C          = 3 -> Z-DIRECTION SEARCH
C          = 4 -> R-DIRECTION SEARCH  (X=Y)
C          = 5 -> GLOBAL-DIRECTION SEARCH (X=Y=Z)
C----
            IF(IHETL .GT. 0 .AND. IOUTER .GT. 0) THEN
              CALL FLUBLN(IPMACR,IPRINT,NGROUP,NBMIX ,NREGIO,NUNKNO,
     >                    MATCOD,VOLUME,B2    ,FUNKNO,SIGT0 ,SIGS0 ,
     >                    IJJ   ,NJJ   ,IPOS  ,XSCAT ,IHETL ,REFKEF,
     >                    BIL1  )
              B2NEW=B2(1)+B2(2)+B2(3)
            ELSE
              B2NEW=B2OLD
            ENDIF
            IF(COPTIO .EQ. 'B1  ' .OR. COPTIO .EQ. 'B0  ' .OR.
     >         COPTIO .EQ. 'B0TR' ) THEN
              CALL FLULBD(IPRINT,NGROUP,B2NEW ,SIGTH ,DB2   )
            ELSE
              CALL XDRSET(DB2,NGROUP,1.0)
            ENDIF
            DO 232 IGR=1,NGROUP
              DO 233 IREG=1,NREGIO
                BIL2=BIL2+DBLE(
     >               (FUNKNO(NREGIO+KEYFLX(IREG),IGR)*B2(1)+
     >               FUNKNO(2*NREGIO+KEYFLX(IREG),IGR)*B2(2)+
     >               FUNKNO(3*NREGIO+KEYFLX(IREG),IGR)*B2(3))
     >                   *VOLUME(IREG))
 233          CONTINUE
 232        CONTINUE
          ENDIF
        ENDIF
C----
C  COMPLETE EVALUATION OF NEUTRON BALANCE
C  CUREIG = TOTAL NEUTRON ABSORPTION
C----
        DO 130 IGR=1,NGROUP
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
          CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
          CALL LCMGET(IPMACR,'TOTAL',SIGT0(1))
          CALL LCMGET(IPMACR,'SCA'//CAT,XSCAT(1))
          CALL LCMGET(IPMACR,'IJJ'//CAT,IJJ(1))
          CALL LCMGET(IPMACR,'NJJ'//CAT,NJJ(1))
          CALL LCMGET(IPMACR,'IPO'//CAT,IPOS(1))
          DO 90 IMIX=1,NBMIX
            DSIGS=0.0D0
            DO 91 IG2=IJJ(IMIX)-NJJ(IMIX)+1,IJJ(IMIX)
              IXSS=IPOS(IMIX)+IJJ(IMIX)-IG2
              DSIGS=DSIGS+DBLE(XSCAT(IXSS))
 91         CONTINUE
            SIGS0(IMIX)=SIGT0(IMIX)-REAL(DSIGS)
 90       CONTINUE
          DO 95 IREG=1,NREGIO
            BIL2=BIL2+DBLE(FUNKNO(KEYFLX(IREG),IGR)
     >               *SIGT0(MATCOD(IREG))*VOLUME(IREG))
            CUREIN=CUREIN+DBLE(FUNKNO(KEYFLX(IREG),IGR)
     >               *SIGS0(MATCOD(IREG))*VOLUME(IREG))
   95     CONTINUE
          CALL LCMGET(IPMACR,'SCA'//CAN,XSCAT(1))
          CALL LCMGET(IPMACR,'IJJ'//CAN,IJJ(1))
          CALL LCMGET(IPMACR,'NJJ'//CAN,NJJ(1))
          CALL LCMGET(IPMACR,'IPO'//CAN,IPOS(1))
          DO 310 IREG=1,NREGIO
            IBM=MATCOD(IREG)
            DO 311 IG2=IJJ(IBM)-NJJ(IBM)+1,IJJ(IBM)
              IXSS=IPOS(IBM)+IJJ(IBM)-IG2
              BIL2=BIL2-DBLE(FUNKNO(KEYFLX(IREG),IG2)
     >                 *XSCAT(IXSS)*VOLUME(IREG))
  311       CONTINUE
  310     CONTINUE
          CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
  130   CONTINUE
        IF(CUREIN .GT. 0.0D0) THEN
          CUREIN=BIL1/CUREIN
        ELSE
          CUREIN=0.0D0
        ENDIF
C----
C  COMPUTE NEW EIGENVALUE AND SOURCE NORMALIZATION FACTOR
C  FACTNO = 1/EIGNEW (LEAKKAGE CASE   -> BIL2>0 AND LEAKSW=.TRUE.)
C  FACTNO = 1/BIL1   (NO LEAKAGE CASE -> BIL2=0 OR  LEAKSW=.FALSE.)
C----
        IF(BIL1 .GT. 0.0D0) THEN
          FACTNO=1.0D0/BIL1
        ELSE
          FACTNO=1.0D0
        ENDIF
        IF(LEAKSW) THEN
          IF(OLDBIL .GT. 0.0D0) THEN
            EIGNEW=BIL1/OLDBIL
          ELSE
            EIGNEW=BIL1
          ENDIF
        ELSE
          IF(BIL2 .GT. 0.0D0) THEN
            EIGNEW=BIL1/BIL2
            FACTNO=FACTNO*BIL2
          ELSE
            EIGNEW=BIL1
          ENDIF
        ENDIF
        IF(EIGNEW .EQ. 0.0D0 .AND.
     >     NIFISS .GT. 0 ) CALL XABORT(NAMSBR//
     >  ': FISSION IS MISSING.')
      ELSE IF(IGENK .EQ. 0) THEN
        CALL FLUQFX(IPMACR,IPRINT,NGROUP,NBMIX ,NREGIO,NUNKNO,
     >              MATCOD,KEYFLX,CFIXSR,SUNKNO,SIGT0 )
        IF (ISADJ .GE. 3) THEN
          FACTNO=1.0D0/REFKEF
        ELSE
          FACTNO=1.0D0
        ENDIF
        IF(NIFISS .GT. 0) THEN
          DO 40 IGR=1,NGROUP
            WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
            CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
            CALL LCMGET(IPMACR,CNUF,SIGF0)
            DO 41 IREG=1,NREGIO
              IBM=MATCOD(IREG)
              IF(IBM .GT. 0) THEN
                DO 42 IFIS=1,NIFISS
                  QTOTL(IREG,IFIS)=QTOTL(IREG,IFIS)
     >               +FUNKNO(KEYFLX(IREG),IGR)*SIGF0(IBM,IFIS)
 42             CONTINUE
              ENDIF
 41         CONTINUE
            CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 40       CONTINUE
        ENDIF
      ENDIF
      IF(NIFISS .GT. 0) THEN
        CALL FLUQFS(IPMACR,IPRINT,NGROUP,NBMIX ,NREGIO,NUNKNO,
     >              NIFISS,MATCOD,KEYFLX,CCHI  ,FACTNO,QTOTL ,
     >              SUNKNO,SIGF0 )
C----
C  ILEAK >= 5 ADD B**2 FLUX SOURCES
C----
        IF(IBUCK .EQ. 2 .AND. ILEAK .GE. 5) THEN
          CALL SETARA(RBASE,3*NREGIO*NREGIO,IPPP)
          CALL FLUQFB(IPSYS ,IPRINT,NGROUP,NREGIO,NUNKNO,B2    ,
     >                FUNKNO,SUNKNO,RBASE(IPPP)  )
          CALL RLSARA(RBASE(IPPP))
        ENDIF
      ENDIF
C----
C  CHECK THE EIGENVALUE CONVERGENCE.
C----
      IF(IGENK .EQ. 0) THEN
        RELEIG=0.0
      ELSE IF(IOUTER .EQ. 0) THEN
        RELEIG=1.0
      ELSE IF(IGENK .EQ. 3) THEN
        RELEIG=ABS(SQRT(ABS(B2NEW))-SQRT(ABS(B2OLD)))/
     >         SQRT(ABS(B2NEW))
      ELSE
        RELEIG=ABS(EIGNEW-EIGENK)/ABS(EIGNEW)
      ENDIF
      LCEIG=RELEIG .LT. EPSEIG
      RELCOU=0.0
      RELFLX=0.0
      CALL SETARA(RBASE,NUNKNO,IDELFL)
      IF(IOUTER .GT. 0 .AND.
     >   IGENK  .EQ. 0 .AND.
     >   NIFISS .GT. 0      ) THEN
C----
C  CHECK THE FLUX CONVERGENCE.
C----
        GRMAX=0
        FIMOY=0.0
        DO 470 IGR=1,NGROUP
          WRITE(CGRPNM,'(4HFLUX,I3)') IGR
          CALL LCMLEN(IPFLUX,CGRPNM,ILCMLN,ILCMTY)
          IF(ILCMLN .NE. 0) THEN
            CALL LCMGET(IPFLUX,CGRPNM,RBASE(IDELFL))
            TDPHI=0.0
            TMPHI=0.0
            MAXREG=0 
            DO 471 IREG=1,NREGIO
              IPOIND=IDELFL+KEYFLX(IREG)-1
              FIMOY=FIMOY+ABS(FUNKNO(KEYFLX(IREG),IGR)) 
              TDPHI=MAX(ABS(FUNKNO(KEYFLX(IREG),IGR)
     >                     -RBASE(IPOIND)),TDPHI)
              IF(ABS(FUNKNO(KEYFLX(IREG),IGR)) .GT. TMPHI) MAXREG=IREG
              TMPHI=MAX(ABS(FUNKNO(KEYFLX(IREG),IGR)),TMPHI)
 471        CONTINUE
            LCFLX=LCFLX .AND. (TDPHI .LT. TMPHI*EPSFLX)
            IF(TMPHI .GT. 0.0) THEN
              IF(TDPHI/TMPHI .GT. RELFLX) GRMAX=IGR
              RELFLX=MAX(RELFLX,TDPHI/TMPHI)
            ENDIF
C----
C  CHECK THE CURRENT CONVERGENCE IF ILEAK >= 5.
C----
          ELSE
            LCFLX=.FALSE.
          ENDIF
          CALL LCMPUT(IPFLUX,CGRPNM,NUNKNO,2,FUNKNO(1,IGR))
 470    CONTINUE
        LOSCIL=(RELFOL .GT. 0 .AND. ISADJ .GE. 3 .AND.
     >         (LOSCIL .OR. RELFLX .GT. RELFOL))    
        FIMOY=FIMOY/REAL(NGROUP*NREGIO)
        FIMOY=ABS(FUNKNO(KEYFLX(MAXREG),GRMAX))/FIMOY
        IF (LOSCIL .AND. FIMOY .LT. 1.0E-4) THEN 
          WRITE(IOUT,350) FIMOY
          LCFLX=.TRUE.
        ENDIF
        RELFOL=RELFLX 
      ELSE IF(IOUTER .GT. 0 .AND.
     >        IGENK  .GT. 0 .AND.
     >        LCEIG                ) THEN
        DO 370 IGR=1,NGROUP
          WRITE(CGRPNM,'(4HFLUX,I3)') IGR
          CALL LCMLEN(IPFLUX,CGRPNM,ILCMLN,ILCMTY)
          IF(ILCMLN .NE. 0) THEN
            CALL LCMGET(IPFLUX,CGRPNM,RBASE(IDELFL))
            TDPHI=0.0
            TMPHI=0.0
            DO 371 IREG=1,NREGIO
              IPOIND=IDELFL+KEYFLX(IREG)-1
              TDPHI=MAX(ABS(FUNKNO(KEYFLX(IREG),IGR)
     >                     -RBASE(IPOIND)),TDPHI)
              TMPHI=MAX(ABS(FUNKNO(KEYFLX(IREG),IGR)),TMPHI)
 371        CONTINUE
            LCFLX=LCFLX .AND. (TDPHI .LT. TMPHI*EPSFLX)
            IF(TMPHI .GT. 0.0) RELFLX=MAX(RELFLX,TDPHI/TMPHI)
C----
C  CHECK THE CURRENT CONVERGENCE IF ILEAK >= 5.
C----
            IF(IBUCK .EQ. 2 .AND. ILEAK .GE. 5) THEN
              TDCOU=0.0
              TMCOU=0.0
              DO 372 IDIR=1,3
                KCOU=IDIR*NREGIO
                DO 373 IREG=1,NREGIO
                  IPOIND=IDELFL+KEYFLX(IREG)-1+KCOU
                  IPOINF=KCOU+KEYFLX(IREG)
 373            CONTINUE
                DO 374 IREG=1,NREGIO
                  IPOIND=IDELFL+KEYFLX(IREG)-1+KCOU
                  IPOINF=KCOU+KEYFLX(IREG)
                  TDCOU=MAX(ABS( FUNKNO(IPOINF,IGR)
     >                 -RBASE(IPOIND)),TDCOU)
                  TMCOU=MAX(ABS(FUNKNO(IPOINF,IGR)),TMCOU)
 374            CONTINUE
 372          CONTINUE
              LCCOU=LCCOU .AND. (TDCOU .LT. TMCOU*EPSFLX)
              IF(TMCOU .GT. 0.0) RELCOU=MAX(RELCOU,TDCOU/TMCOU)
            ENDIF
          ELSE
            LCFLX=.FALSE.
            IF(IBUCK .EQ. 2 .AND. ILEAK .GE. 5 ) LCCOU=.FALSE.
          ENDIF
          CALL LCMPUT(IPFLUX,CGRPNM,NUNKNO,2,FUNKNO(1,IGR))
 370    CONTINUE
      ELSE
        CALL FLUSFL(IPFLUX,NGROUP,NUNKNO,FUNKNO)
      ENDIF
      CALL RLSARA(RBASE(IDELFL))
      LCONV=LCFLX.AND.LCCOU.AND.LCEIG
      IF(IOUTER .GT. 0.AND.
     >   IPRINT .GE. 1) THEN
        WRITE(IOUT,400) HTYPE(IGENK),IOUTER
        IF(IGENK .EQ. 1) THEN
          IF(IBUCK .EQ. 0) THEN
            WRITE(IOUT,410) EIGNEW,CUREIN
          ELSE
            IF(ILEAK .LT. 5) THEN
              WRITE(IOUT,420) EIGNEW,CUREIN,B2(4)
            ELSE
              WRITE(IOUT,421) EIGNEW,CUREIN,B2NEW,(B2(JJ),JJ=1,3)
            ENDIF
          ENDIF
        ELSE IF(IGENK .GE. 2) THEN
          IF(ILEAK .LT. 5) THEN
            WRITE(IOUT,430) B2(4),EIGNEW,CUREIN,REFKEF
          ELSE
            IF(ILEAK .EQ. 6) THEN
              WRITE(IOUT,431) B2NEW,
     >            'COMPUTED',B2(1),'IMPOSED ',B2(2),'IMPOSED ',B2(3),
     >             EIGNEW,CUREIN,REFKEF
            ELSE IF(ILEAK .EQ. 7) THEN
              WRITE(IOUT,431) B2NEW,
     >            'IMPOSED ',B2(1),'COMPUTED',B2(2),'IMPOSED ',B2(3),
     >             EIGNEW,CUREIN,REFKEF
            ELSE IF(ILEAK .EQ. 8) THEN
              WRITE(IOUT,431) B2NEW,
     >            'IMPOSED ',B2(1),'IMPOSED ',B2(2),'COMPUTED',B2(3),
     >             EIGNEW,CUREIN,REFKEF
            ELSE IF(ILEAK .EQ. 9) THEN
              WRITE(IOUT,432) B2NEW,
     >        'COMPUTED',B2(1)+B2(2),
     >        'COMPUTED',B2(1),'COMPUTED',B2(2),'IMPOSED ',B2(3),
     >        EIGNEW,CUREIN,REFKEF
            ELSE IF(ILEAK .EQ. 10) THEN
              WRITE(IOUT,431) B2NEW,'COMPUTED',B2(1),'COMPUTED',B2(2),
     >        'COMPUTED',B2(3),EIGNEW,CUREIN,REFKEF
            ELSE IF(ILEAK .EQ. 11) THEN
              WRITE(IOUT,433) B2NEW,'COMPUTED',B2(1),'COMPUTED',B2(2),
     >        'COMPUTED',B2(3),EIGNEW,CUREIN,REFKEF
            ENDIF
          ENDIF
        ENDIF
        IF(LCEIG) THEN
          IF     (IGENK .EQ. 0) THEN
            WRITE(IOUT,444) RELFLX
          ELSE IF( IBUCK .EQ. 0) THEN
            WRITE(IOUT,440) RELEIG,RELFLX
          ELSE
            IF(ILEAK .LT. 5) THEN
              WRITE(IOUT,440) RELEIG,RELFLX
            ELSE
              WRITE(IOUT,441) RELEIG,RELFLX,RELCOU
            ENDIF
          ENDIF
        ELSE
          IF(IGENK .GE. 1) THEN
            IF(IBUCK .EQ. 0) THEN
              WRITE(IOUT,442) RELEIG
            ELSE
              IF(ILEAK .LT. 5) THEN
                WRITE(IOUT,442) RELEIG
              ELSE
                WRITE(IOUT,443) RELEIG
              ENDIF
            ENDIF
          ENDIF
        ENDIF
      ENDIF
      EPSFLX=RELFLX
      IF(IGENK .GT. 0) THEN
        EIGENK=EIGNEW
        EPSEIG=RELEIG
      ENDIF
      RETURN
C----
C FORMATS
C----
  350 FORMAT(1X,'*** WARNING *** OSCILLATIONS WITH FLUXES CLOSE',
     >       ' TO ZERO ',/
     >       1X,'RATIO HIGHEST FLUX IN THE MOST DIVERGENT',
     >       ' GROUP - AVERAGE FLUX: ',1P,E11.3,//
     >       14X,' ---CONVERGENCE FORCED---'//)
  400 FORMAT(' TYPE ',A4,' -- OUTER ITERATION COMPLETED  = ',I10)
  410 FORMAT(14X,'CURRENT K-EFFECTIVE        = ',0P,F10.6/
     >       14X,'CURRENT K-INFINITE         = ',F10.6)
  420 FORMAT(14X,'CURRENT K-EFFECTIVE        = ',0P,F10.6/
     >       14X,'CURRENT K-INFINITE         = ',F10.6/
     >       14X,'IMPOSED TOTAL BUCKLING     = ',1P,E13.5)
  421 FORMAT(14X,'CURRENT K-EFFECTIVE        = ',0P,F10.6/
     >       14X,'CURRENT K-INFINITE         = ',F10.6/
     >       14X,'TOTAL BUCKLING             = ',1P,E13.5/
     >       14X,'IMPOSED X- BUCKLING        = ',E13.5/
     >       14X,'IMPOSED Y- BUCKLING        = ',E13.5/
     >       14X,'IMPOSED Z- BUCKLING        = ',E13.5)
  430 FORMAT(14X,'TOTAL BUCKLING             = ',1P,E13.5/
     >       14X,'CURRENT K-EFFECTIVE        = ',0P,F10.6/
     >       14X,'CURRENT K-INFINITE         = ',F10.6/
     >       14X,'IMPOSED K-EFFECTIVE        = ',F10.6)
  431 FORMAT(14X,'TOTAL BUCKLING             = ',1P,E13.5/
     >       14X,A8     ,' X- BUCKLING       = ',E13.5/
     >       14X,A8     ,' Y- BUCKLING       = ',E13.5/
     >       14X,A8     ,' Z- BUCKLING       = ',E13.5/
     >       14X,'CURRENT K-EFFECTIVE        = ',0P,F10.6/
     >       14X,'CURRENT K-INFINITE         = ',F10.6/
     >       14X,'IMPOSED K-EFFECTIVE        = ',F10.6)
  432 FORMAT(14X,'TOTAL BUCKLING             = ',1P,E13.5/
     >       14X,A8     ,' RADIAL BUCKLING   = ',E13.5/
     >       14X,A8     ,' X- BUCKLING       = ',E13.5/
     >       14X,A8     ,' Y- BUCKLING       = ',E13.5/
     >       14X,A8     ,' Z- BUCKLING       = ',E13.5/
     >       14X,'CURRENT K-EFFECTIVE        = ',0P,F10.6/
     >       14X,'CURRENT K-INFINITE         = ',F10.6/
     >       14X,'IMPOSED K-EFFECTIVE        = ',F10.6)
  433 FORMAT(14X,'TOTAL BUCKLING             = ',1P,E13.5/
     >       14X,'RATIO ',A8  ,' X- BUCKLING = ',E13.5/
     >       14X,'RATIO ',A8  ,' Y- BUCKLING = ',E13.5/
     >       14X,'RATIO ',A8  ,' Z- BUCKLING = ',E13.5/
     >       14X,'CURRENT K-EFFECTIVE        = ',0P,F10.6/
     >       14X,'CURRENT K-INFINITE         = ',F10.6/
     >       14X,'IMPOSED K-EFFECTIVE        = ',F10.6)
  440 FORMAT(14X,'ERROR ON EIGENVALUE        = ',1P,E11.3/
     >       14X,'MAXIMUM ERROR ON FLUXES    = ',E11.3)
  441 FORMAT(14X,'ERROR ON EIGENVALUE        = ',1P,E11.3/
     >       14X,'MAXIMUM ERROR ON FLUXES    = ',E11.3/
     >       14X,'MAXIMUM ERROR ON CURRENTS  = ',E11.3)
  442 FORMAT(14X,'ERROR ON EIGENVALUE        = ',1P,E11.3/
     >       14X,'MAXIMUM ERROR ON FLUXES    =  NOT CHECKED')
  443 FORMAT(14X,'ERROR ON EIGENVALUE        = ',1P,E11.3/
     >       14X,'MAXIMUM ERROR ON FLUXES    =  NOT CHECKED'/
     >       14X,'MAXIMUM ERROR ON CURRENTS  =  NOT CHECKED')
  444 FORMAT(14X,'MAXIMUM ERROR ON FLUXES    = ',1P,E11.3)
      END
