*DECK MAC
      SUBROUTINE MAC(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
* MACROSCOPIC CROSS SECTIONS AND DIFFUSION COEFFICIENTS INPUT MODULE
* FOR DRAGON.
*
* INPUT/OUTPUT PARAMETERS:
*  NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
*  HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
*  IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
*           =4 SEQUENTIAL ASCII FILE.
*  JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
*           =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
*           =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
*  KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.
*           DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
*           KENTRY(NENTRY)
*
* LINKED LIST / XSM FILE:
*  HENTRY(1) : CREATE OR MODIFICATION MACROLIB.
*  HENTRY(2) : OLD MACROLIB.
*
*-------------------------------------- AUTHOR: A. HEBERT ; 27/09/94 ---
* MODIFICATIONS:
*   2000/02/17    T. COURAU + G. MARLEAU
*                 ADD ADJOINT SCATTERING INFORMATION
*   1998/06/03    G. MARLEAU
*                 INSERT OPTION TO UPDATE/CREATE A MACROLIB
*                 FROM A SET OF OTHER MACROLIB
*-----------------------------------------------------------------------
*
      IMPLICIT     NONE
      INTEGER      IOUT,NSTATE,NTC,ILCMUP,ILCMDN
      CHARACTER    NAMSBR*6,NAMMOD*12
      PARAMETER   (IOUT=6,NSTATE=20,NTC=3,ILCMUP=1,ILCMDN=2,
     >             NAMSBR='MAC   ',NAMMOD='MAC:        ')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER      NENTRY
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IMLOC
C----
C  INPUT
C----
      CHARACTER    CARLIR*12
      INTEGER      ITYPLU,INTLIR
      REAL         REALIR
      DOUBLE PRECISION DBLLIR
C----
C  LOCAL PARAMETERS
C----
      INTEGER      ISTATE(NSTATE),KCHAR(NTC),ITYPE,IPMACR,IPRINT,
     >             NMACSR,NOLDMX,NGO,NLO,NFO,NEO,ITO,ITC,
     >             INDREC,NGROUP,NBMIX,NANISO,NIFISS,NNEWMX,
     >             NEDMAC,ITRANC,NALBD,NTOTMX,IPMAC2,JJ
      CHARACTER    TEXT12*12,HSIGN*12
C----
C  PRINT CREDITS
C----
      WRITE(IOUT,6900) NAMMOD
      WRITE(IOUT,6910)
*----
* PARAMETER VALIDATION.
*----
      IF(NENTRY .LT. 1) CALL XABORT(NAMSBR//
     > ': AT LEAST ONE DATA STRUCTURE REQUIRED')
      IF(IENTRY(1) .NE. 1 .AND. IENTRY(1) .NE. 2) CALL XABORT(NAMSBR//
     > ': FIRST DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
      IF(JENTRY(1) .NE. 0 .AND. JENTRY(1) .NE. 1) CALL XABORT(NAMSBR//
     > ': FIRST DATA STRUCTURE NOT IN CREATE OR MODIFY MODE')
      ITYPE=JENTRY(1)
      IPMACR=KENTRY(1)
      NGO=0
C----
C  START MODIFICATIONS 98/06/03
C  1) LOOK FOR OTHER MACROLIB IN SET OF DATA STRUCTURES
C----
      NMACSR=1
      IF(NENTRY .GT. 2) THEN
       CALL XABORT(NAMSBR//': ONLY TWO MACROLIB NOW PERMITTED')
      ELSE IF(NENTRY .EQ. 2) THEN
        IPMAC2=NENTRY
        IF(IENTRY(IPMAC2) .NE. 1 .AND.
     >     IENTRY(IPMAC2) .NE. 2) CALL XABORT(NAMSBR//
     >  ': INVALID STRUCTURE TYPE FOR MACROLIB ')
        IF(JENTRY(IPMAC2) .NE. 2) CALL XABORT(NAMSBR//
     >  ': DATA STRUCTURE '//HENTRY(IPMAC2)//'NOT IN READONLY MODE')
        CALL LCMGET(KENTRY(IPMAC2),'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
        IF(HSIGN .EQ. 'L_MACROLIB') THEN
          NMACSR=2
          CALL LCMGET(KENTRY(IPMAC2),'STATE-VECTOR',ISTATE)
          IF(ISTATE(2) .GT. 0) THEN
            NOLDMX=ISTATE(2)
            NGO=ISTATE(1)
            NLO=ISTATE(3)
            NFO=ISTATE(4)
            NEO=ISTATE(5)
            ITO=ISTATE(6)
          ELSE
            NMACSR=1
          ENDIF
        ELSE IF(HSIGN .EQ. 'L_LIBRARY') THEN
          NMACSR=-2
          CALL LCMSIX(KENTRY(IPMAC2),'MACROLIB',ILCMUP)
          CALL LCMGET(KENTRY(IPMAC2),'STATE-VECTOR',ISTATE)
          IF(ISTATE(2) .GT. 0) THEN
            NOLDMX=ISTATE(2)
            NGO=ISTATE(1)
            NLO=ISTATE(3)
            NFO=ISTATE(4)
            NEO=ISTATE(5)
            ITO=ISTATE(6)
          ELSE
            CALL LCMSIX(KENTRY(IPMAC2),'MACROLIB',ILCMDN)
            NMACSR=1
          ENDIF
        ELSE
          CALL XABORT(NAMSBR//
     >        ': SECOND DATA STRUCTURE HAS INVALID SIGNATURE')
        ENDIF
      ENDIF
C----
C  READ THE INPUT DATA.
C     DEFAULT OPTIONS:
C----
      IPRINT=1
      CALL XDRSET(ISTATE,NSTATE,0)
      IF(ITYPE .EQ. 0) THEN
        INDREC=1
        NGROUP=0
        NBMIX =0
        NANISO=1
        NIFISS=0
        NEDMAC=0
        ITRANC=0
        NALBD =0
        HSIGN='L_MACROLIB'
        READ(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
        CALL LCMPUT(IPMACR,'SIGNATURE',3,3,KCHAR)
      ELSE IF(ITYPE .EQ. 1) THEN
        CALL LCMGET(IPMACR,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
        IF(HSIGN .EQ. 'L_MACROLIB') THEN
          INDREC=2
          CALL LCMGET(IPMACR,'STATE-VECTOR',ISTATE)
        ELSE IF(HSIGN .EQ. 'L_LIBRARY') THEN
          INDREC=3
          CALL LCMSIX(IPMACR,'MACROLIB',ILCMUP)
          CALL LCMGET(IPMACR,'STATE-VECTOR',ISTATE)
        ELSE
          TEXT12=HENTRY(1)
          CALL XABORT(NAMSBR//
     >    ': SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     >    '. L_MACROLIB OR L_LIBRARY EXPECTED.')
        ENDIF
        NGROUP=ISTATE(1)
        NBMIX =ISTATE(2)
        NANISO=ISTATE(3)
        NIFISS=ISTATE(4)
        NEDMAC=ISTATE(5)
        ITRANC=ISTATE(6)
        NALBD =ISTATE(8)
      ENDIF
      IF(NMACSR .EQ. 1) THEN
C----
C  TAKE MACROSCOPIC XS FROM INPUT OR GOXS FILE
C----
        CALL MACDRV(IPMACR,INDREC,IPRINT,NBMIX ,NGROUP,NANISO,
     >              NIFISS,NEDMAC,ITRANC,NALBD )
      ELSE
        NNEWMX=0
        NANISO=MAX(NLO,NANISO)
        NIFISS=NFO+NIFISS
        NEDMAC=MAX(NEDMAC,NEO)
        ITRANC=MAX(ITRANC,ITO)
C----
C  TAKE MACROSCOPIC XS FROM OLD MACROLIB
C  READ MAIN INPUT PARAMETERS UNTIL KEYWORD MIX FOUND
C----
 1000   CONTINUE
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': CHARACTER KEYWORD EXPECTED')
C----
C  CHECK FOR STOP/RETURN
C----
        IF(CARLIR .EQ. 'EDIT') THEN
C----
C  READ THE PRINT INDEX
C----
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >    ': EDIT LEVEL SHOULD BE AN INTEGER')
          IPRINT=INTLIR
        ELSE IF(CARLIR .EQ. 'NMIX') THEN
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >    ': READ ERROR - NUMBER OF MIXTURES EXPECTED')
            NNEWMX=MAX(INTLIR,NNEWMX)
        ELSE IF(CARLIR .EQ. 'ANIS') THEN
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >    ': READ ERROR - ANIS LEVEL EXPECTED')
          NANISO=MAX(NANISO,INTLIR)
        ELSE IF(CARLIR .EQ. 'NIFI') THEN
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//
     >    ': READ ERROR - NUMBER FISSILE ISOTOPES EXPECTED')
          NIFISS=MAX(INTLIR,NIFISS)
        ELSE IF(CARLIR(1:4) .EQ. 'CTRA') THEN
C----
C  READ THE PRINT INDEX
C----
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >    ': CTRA MUST BE FOLLOWED BY CHARACTER')
          IF(CARLIR .EQ. 'OFF') THEN
            ITRANC = 0
          ELSE IF(CARLIR .EQ. 'ON') THEN
            ITRANC = 2
          ELSE
            CALL XABORT(NAMSBR//': INVALID CTRA OPTION')
          ENDIF
        ELSE IF(CARLIR(1:3) .EQ. 'MIX') THEN
          GO TO 1005
        ELSE
          CALL XABORT(NAMSBR//
     >    ': KEYWORD '//CARLIR//' NOT PERMITTED')
        ENDIF
        GO TO 1000
 1005   CONTINUE
        NTOTMX=NOLDMX+NBMIX+NNEWMX
        IF(NGROUP .EQ. 0) THEN
          IF(NGO .EQ. 0 ) CALL XABORT(NAMSBR//
     >    ': MACROLIBS HAVE 0 GROUP ')
          NGROUP=NGO
        ELSE IF(NGROUP .NE. NGO) THEN
          CALL XABORT(NAMSBR//
     >    ': MACROLIBS HAVE DIFFERENT GROUP STRUCTURES')
        ENDIF
        CALL SETARA(IBASE,2*NTOTMX,IMLOC)
        CALL XDRSET(IBASE(IMLOC),2*NTOTMX,0)
        CALL MACUPD(NENTRY,KENTRY,IPRINT,NTOTMX,NBMIX ,NGROUP,
     >              NANISO,NIFISS,NEDMAC,ITRANC,
     >              IBASE(IMLOC) )
        CALL RLSARA(IBASE(IMLOC))
        IF(NMACSR .EQ. -2) THEN
          CALL LCMSIX(KENTRY(IPMAC2),'MACROLIB',ILCMDN)
        ENDIF
      ENDIF
      ISTATE( 1)=NGROUP
      ISTATE( 2)=NBMIX
      ISTATE( 3)=NANISO
      ISTATE( 4)=NIFISS
      ISTATE( 5)=NEDMAC
      ISTATE( 6)=ITRANC
      ISTATE( 7)=0
      ISTATE( 8)=NALBD 
      CALL LCMPUT(IPMACR,'STATE-VECTOR',NSTATE,1,ISTATE)
      IF(INDREC .EQ. 3) THEN
        CALL LCMSIX(IPMACR,'MACROLIB',ILCMDN) 
      ENDIF
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6000) 9,NAMSBR,(ISTATE(JJ),JJ=1,9)
      ENDIF
      IF(IPRINT .GE. 10) CALL LCMLIB(IPMACR)
      WRITE(IOUT,6901) NAMMOD
      RETURN
C----
C  FORMATS
C----
 6000 FORMAT(1X,'MACROLIB STATE-VECTOR (',I2,') ELEMENTS FROM: ',A6,
     >       /1X,32('-')/
     > ' NGROUP =',I8,' (NUMBER OF GROUPS)'/
     > ' NBMIX  =',I8,' (NUMBER OF MIXTURE)'/
     > ' NANISO =',I8,' (MAXIMUM SCATTERING ANISOTROPY)'/
     > ' NIFISS =',I8,' (NUMBER OF FISSILE ISOTOPES)'/
     > ' NEDMAC =',I8,' (NUMBER OF CROSS SECTION EDIT)'/
     > ' ITRANC =',I8,' (TYPE OF TRANSPORT CORRECTION;'/
     >            17X,'  = 0: NO CORRECTION;'/
     >            17X,'  = 1: FROM PN SCATTERING MATRIX;'/
     >            17X,'  = 2: RECOVERED FROM MACROLIB)'/
     > ' NLG    =',I8,' (NUMBER OF DELAYED PRECURSOR GROUPS)'/
     > ' NALBD  =',I8,' (NUMBER OF PHYSICAL ALBEDOS)'/
     > ' ILEAK  =',I8,' (LEAKAGE TYPE)'/
     >        1X,32('-')/)
 6900 FORMAT('->@BEGIN MODULE : ',A12)
 6901 FORMAT('->@END MODULE   : ',A12)
 6910 FORMAT('->@DESCRIPTION  : MACROSCOPIC X-S PROCESSOR'/
     >       '->@CREDITS      : G. MARLEAU'/
     >       '->@COPYRIGHTS   : ECOLE POLYTECHNIQUE DE MONTREAL ')
      END
