*DECK XELDCL
      SUBROUTINE XELDCL( IPGEOM, GEONAM,   NDIM, MAXGRI, LCLSYM, NBLOCK,
     >                    NTYPO,    LL1,    LL2,   IPRT, NTOTCO,  MAXRO,
     >                   NGEOME,   NTYP,  NGIDL,  NTIDL,  NUNKO,  CELLG,
     >                    NSURO,  NVOLO, IDLDIM, IDLGEO, KEYTRN, KEYGEO,
     >                   IDLTYP, KEYTYP, MRGCEL, IDLBLK, ITGEOM,  CELLT,
     >                    ISPLT )
************************************************************************
*                                                                      *
*           NAME: XELDCL                                               *
*      COMPONENT: EXCELL                                               *
*          LEVEL: 3 (CALLED BY 'XELTRK')                               *
*        VERSION: 1.0                                                  *
*       CREATION: 87/01                                                *
*       MODIFIED: 00/03 (R.R.) DECLARE ALL VARIABLE TYPES              *
*         AUTHOR: ROBERT ROY                                           *
*                                                                      *
*     SUBROUTINE: THIS ROUTINE WILL ASSOCIATE ALL BLOCKS OF A PROBLEM  *
*                 TO THEIR BLOCK TYPES & WILL GENERATE ALMOST ALL      *
*                 USEFUL INTEGER VALUES THAT WILL DESCRIBE THE PROBLEM.*
*                                                                      *
*--------+-------------- V A R I A B L E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                 /IO/MOD(DIMENS)*
*--------+----------------------------------------------+--+-----------*
* IPGEOM / POINTER TO THE GEOMETRY (L_GEOM)             /I./INT        *
* GEONAM / GEOMETRY NAME                                /I./CAR*12     *
* NDIM   / # OF DIMENSIONS.                             /I./INT        *
* MAXGRI / # OF GRID CELL IN X/Y/Z DIRECTIONS           /I./INT(3)     *
* LCLSYM / SYMETRY FACTORS.                             /I./INT(3)     *
* NBLOCK / # OF BLOCKS.                                 /I./INT        *
* NTYPO  / OLD # OF TYPES.                              /I./INT        *
* LL1    / UPPER DIAG SWITCH.                           /I./LOG        *
* LL2    / LOWER DIAG SWITCH.                           /I./LOG        *
* IPRT   / INTERMEDIATE PRINTING LEVEL FOR OUTPUT.      /I./INT        *
* NTOTCO / TOT # OF CYLINDERS IN ALL GEOMETRIES         /.O/INT        *
* MAXRO  / MAX # OF WORDS TO STOCK MESHES.              /.O/INT        *
* NGEOME / # OF GEOMETRIES.                             /.O/INT        *
* NTYP   / NEW # OF TYPES.                              /.O/INT        *
* NGIDL  / LENGHT OF GEOMETRIC NUMBERING.               /.O/INT        *
* NTIDL  / LENGHT OF TYPE NUMBERING.                    /.O/INT        *
* NUNKO  / OLD # OF UNKNOWNS.                           /.O/INT        *
* CELLG  / TO KEEP CELL GEOMETRY NAMES.                 /.O/C*4        *
* NSURO  / # OF SURFACES OF EACH GEOMETRY.              /.O/INT(NBLOCK)*
* NVOLO  / # OF ZONES OF EACH GEOMETRY.                 /.O/INT(NBLOCK)*
* IDLDIM / POSITION OF EACH GEOEMTRY IN CYLINDERS #ING. /.O/INT(NBLOCK)*
* IDLGEO / POSITION OF EACH GEOMETRY IN THE             /.O/INT(NBLOCK)*
*        /            GEOMETRY NUMBERING SCHEME.        /  /           *
* KEYTRN / TURN KEY FOR EACH BLOCK.                     /.O/INT(NBLOCK)*
* KEYGEO / GEOMETRIC KEY FOR EACH TYPE.                 /.O/INT(NBLOCK)*
* IDLTYP / POSITION OF EACH TYPE IN NUMBERING SCHEME.   /.O/INT(NBLOCK)*
* KEYTYP / TYPE KEY FOR EACH BLOCK.                     /.O/INT(NBLOCK)*
* MRGCEL / MERGING KEY OF EACH BLOCK.                   /.O/INT(NBLOCK)*
* IDLBLK / POSITION OF EACH BLOCK IN NUMBERING SCHEME.  /.O/INT(NBLOCK)*
* ITGEOM / TURN BY CELL TYPES (WORK AREA).              /../INT(NBLOCK)*
* CELLT  / CELL TYPE NAMES (WORK AREA).                 /../C*4(3*NTYPO*
* ISPLT  / SPLITTING VECTOR (WORK AREA).                /../INT(MREGIO)*
*--------+---------------- R O U T I N E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                                *
*--------+-------------------------------------------------------------*
* XELPRC / TO GET MAIN DIMENSIONING FOR EACH CELL.                     *
************************************************************************
C
      IMPLICIT           NONE
C
      INTEGER            IPGEOM, NDIM, NBLOCK, NTYPO, IPRT, NTOTCO,
     >                   MAXRO, NGEOME, NTYP, NGIDL, NTIDL, NUNKO
      LOGICAL            LL1, LL2
      INTEGER             NSURO(NBLOCK),  NVOLO(NBLOCK), IDLDIM(NBLOCK),
     >                   IDLGEO(NBLOCK), KEYGEO(NBLOCK), IDLTYP(NBLOCK),
     >                   KEYTYP(NBLOCK), ITGEOM(NBLOCK), KEYTRN(NBLOCK),
     >                   MRGCEL(NBLOCK), IDLBLK(NBLOCK), ISPLT(*),
     >                   LCLSYM(3), MAXGRI(3), MINGRI(3), MEDGRI(3),
     >                   CELLT(3*NTYPO), CELLG(3*NBLOCK)
      CHARACTER          GEONAM*12,GEOC1*12,GEOC2*12,BLANC*8,CPLAN*8
      LOGICAL            LLSYM
      INTEGER            NSTATE, IOUT, MAXTUR
      PARAMETER        ( NSTATE=20, IOUT=6, MAXTUR=12 )
      INTEGER            ISTATE(NSTATE), ITURN(2*MAXTUR,3)
      CHARACTER*2        CTURN(2*MAXTUR)
      CHARACTER          GEOCV*12
      INTEGER            IKG,IKT(2)
      LOGICAL            LELCSY
C
      INTEGER            IBLK, I3, ISUB2, ITYP, JTYP, K, IX, IY, IZ,
     >                   NMERG1, NMERG2, NMERG3, IOFF, IOF1, IOF2,
     >                   IMERG1, NNCYL, NNSUR, NNVOL, NXC, NXM, IGEO,
     >                   IB1, IB2, IC1, IC2, IT1, IT2, IR, NLINP,
     >                   NTYP2, IOLTYP, NPROB, IDLPRB, IP, MAXREM
C
C                         DEFINITION OF TURNS,  '-' INDICATES A Z-TURN.
      DATA CTURN, ITURN /' A',' B',' C',' D',' E',' F',' G',' H',
     >                   ' I',' J',' K',' L',
     >                   '-A','-B','-C','-D','-E','-F','-G','-H',
     >                   '-I','-J','-K','-L',
C                         TURNS FOR SYMMETRY IN *X*
     >                     5 ,  8 ,  7 ,  6 ,  1 ,  4 ,  3 ,  2 ,
     >                     0 ,  0 ,  0 ,  0 ,
     >                    17 , 20 , 19 , 18 , 13 , 16 , 15 , 14 ,
     >                     0 ,  0 ,  0 ,  0 ,
C                         TURNS FOR SYMMETRY IN *Y*
     >                     7 ,  6 ,  5 ,  8 ,  3 ,  2 ,  1 ,  4 ,
     >                     0 ,  0 ,  0 ,  0 ,
     >                    19 , 18 , 17 , 20 , 15 , 14 , 13 , 16 ,
     >                     0 ,  0 ,  0 ,  0 ,
C                         TURNS FOR DIAGONAL SYMMETRY IN *X-Y*
     >                     6 ,  5 ,  8 ,  7 ,  2 ,  1 ,  4 ,  3 ,
     >                     0 ,  0 ,  0 ,  0 ,
     >                    18 , 17 , 20 , 19 , 14 , 13 , 16 , 15 ,
     >                     0 ,  0 ,  0 ,  0 /
      DATA BLANC        / ' ' /
C
      DO 20 IBLK= 1, NBLOCK
           NSURO(IBLK)= 0
           NVOLO(IBLK)= 0
          IDLGEO(IBLK)= 0
          KEYGEO(IBLK)= 0
          IDLTYP(IBLK)= 0
          KEYTYP(IBLK)= 0
          ITGEOM(IBLK)= 0
          KEYTRN(IBLK)= 0
          IDLBLK(IBLK)= 0
   20 CONTINUE
      LLSYM=.FALSE.
      DO 30 I3= 1, 3
         MINGRI(I3)= MAXGRI(I3)
         MEDGRI(I3)= 0
         IF( LCLSYM(I3).NE.0 )THEN
            MINGRI(I3)= (MAXGRI(I3)+1)/2
            LLSYM=.TRUE.
            IF( LCLSYM(I3).EQ.-1 )THEN
               MEDGRI(I3)= MINGRI(I3)-1
            ELSEIF( LCLSYM(I3).NE.1 )THEN
               WRITE(IOUT,'(1H0,A6,4H -->,3(I6,1X))') 'LCLSYM', LCLSYM
               CALL XABORT( 'XELDCL: LCLSYM NOT WELL DEFINED' )
            ENDIF
         ENDIF
   30 CONTINUE
C
      CALL XDRSET(ISTATE,NSTATE,0)
      CALL LCMGET(IPGEOM,'STATE-VECTOR',ISTATE)
      ISUB2= ISTATE(9)
C
      MAXRO = 0
      NTOTCO= 0 
      IF( ISUB2.GT.0 )THEN
C----GM
      CALL LCMGET(IPGEOM,'CELL',CELLT)
C----GM
C
C        WHEN THERE ARE CELLS IN THE GEOMETRY
         CALL LCMLEN(IPGEOM,'GENERATING', NMERG1, ITYP)
         CALL LCMGET(IPGEOM,'GENERATING', KEYTYP)
C
         CALL LCMLEN(IPGEOM,'MERGE', NMERG2, ITYP)
         IF( NMERG2.EQ.0 )THEN
            DO 55 IMERG1= 1, NMERG1
               MRGCEL(IMERG1)= IMERG1
   55       CONTINUE
         ELSEIF( NMERG2.EQ.NMERG1 )THEN
            CALL LCMGET(IPGEOM,'MERGE', MRGCEL)
         ELSE
            CALL XABORT( ' XELDCL: MERGES ARE INCOMPATIBLE' )
         ENDIF
C
         CALL LCMLEN(IPGEOM,'TURN', NMERG3, ITYP)
         IF( NMERG3.EQ.0 )THEN
            DO 65 IMERG1= 1, NMERG1
               ITGEOM(IMERG1)= 1
   65       CONTINUE
         ELSEIF( NMERG3.EQ.NMERG1 )THEN
            CALL LCMGET(IPGEOM,'TURN', ITGEOM)
            DO 66 IMERG1= 1, NMERG3
               IF( MOD(ITGEOM(IMERG1),MAXTUR).EQ.0.OR.
     >             MOD(ITGEOM(IMERG1),MAXTUR).GT.8 )
     >         CALL XABORT( ' XELDCL: INVALID TURNS (NO HEX CODES)' )
   66       CONTINUE
         ELSE
            CALL XABORT( ' XELDCL: TURNS ARE INCOMPATIBLE' )
         ENDIF
C
C        FILL UP "KEYTYP" ARRAY IN THE CASE OF DIAGONAL SYMMETRY
         IF( LL1 )THEN
            K=ISTATE(6)
            DO 70 IZ=MINGRI(3),1,-1
            IOFF=(IZ-1)*MINGRI(1)*MINGRI(2)
            DO 70 IY=MINGRI(2),1,-1
            DO 60 IX=MINGRI(1),IY+1,-1
            KEYTYP(IOFF+(IY-1)*MINGRI(1)+IX)=
     >                    KEYTYP(IOFF+(IX-1)*MINGRI(2)+IY)
            ITGEOM(IOFF+(IY-1)*MINGRI(1)+IX)=
     >                ITURN(ITGEOM(IOFF+(IX-1)*MINGRI(2)+IY),3)
   60       CONTINUE
            DO 70 IX=IY,1,-1
            KEYTYP(IOFF+(IY-1)*MINGRI(1)+IX)=KEYTYP(K)
            ITGEOM(IOFF+(IY-1)*MINGRI(1)+IX)=ITGEOM(K)
C----GM
      IOF1=K
      IOF2=IOFF+(IY-1)*MINGRI(1)+IX
      IF(IX .EQ. IY) THEN
        IKG=KEYTYP(IOF1)
        IKT(1)=ITGEOM(IOF1)
        IKT(2)=ITURN(IKT(1),3)
        WRITE(GEOCV,'(3A4)')
     >  CELLT(3*IKG-2),CELLT(3*IKG-1),CELLT(3*IKG)
        IF(IPRT .GT. 10)        
     >    WRITE(IOUT,8000) 'X-Y',GEOCV,
     >    CTURN(IKT(1)),CTURN(IKT(2))
        IF(.NOT. LELCSY(IPGEOM,GEOCV,IKT) ) THEN 
          WRITE(IOUT,8001) 'X-Y',GEOCV,
     >    CTURN(IKT(1)),CTURN(IKT(2))
          CALL XABORT('XELDCL: INVALID SYMMETRY FOR CELL')
        ENDIF
      ENDIF
C----GM
   70       K=K-1
            IF (K.NE.0)
     >        CALL XABORT( 'XELDCL: DATA ERROR')
         ELSEIF( LL2 )THEN
            K=ISTATE(6)
            DO 80 IZ=MINGRI(3),1,-1
            IOFF=(IZ-1)*MINGRI(1)*MINGRI(2)
            DO 80 IY=MINGRI(2),1,-1
            DO 80 IX=MINGRI(1),IY,-1
            KEYTYP(IOFF+(IY-1)*MINGRI(1)+IX)=KEYTYP(K)
            ITGEOM(IOFF+(IY-1)*MINGRI(1)+IX)=ITGEOM(K)
C----GM
      IOF1=K
      IOF2=IOFF+(IY-1)*MINGRI(1)+IX
      IF(IX .EQ. IY) THEN
        IKG=KEYTYP(IOF1)
        IKT(1)=ITGEOM(IOF1)
        IKT(2)=ITURN(IKT(1),3)
        WRITE(GEOCV,'(3A4)')
     >  CELLT(3*IKG-2),CELLT(3*IKG-1),CELLT(3*IKG)       
        IF(IPRT .GT. 10)        
     >    WRITE(IOUT,8000) 'X-Y',GEOCV,
     >    CTURN(IKT(1)),CTURN(IKT(2))
        IF(.NOT. LELCSY(IPGEOM,GEOCV,IKT) ) THEN 
          WRITE(IOUT,8001) 'X-Y',GEOCV,
     >    CTURN(IKT(1)),CTURN(IKT(2))
          CALL XABORT('XELDCL: INVALID SYMMETRY FOR CELL')
        ENDIF
      ENDIF
C----GM
   80       K=K-1
            DO 90 IZ=1,MINGRI(3)
            IOFF=(IZ-1)*MINGRI(1)*MINGRI(2)
            DO 90 IY=1,MINGRI(2)
            DO 90 IX=1,IY-1
            KEYTYP(IOFF+(IY-1)*MINGRI(1)+IX)=
     >               KEYTYP(IOFF+(IX-1)*MINGRI(2)+IY)
            ITGEOM(IOFF+(IY-1)*MINGRI(1)+IX)=
     >           ITURN(ITGEOM(IOFF+(IX-1)*MINGRI(2)+IY),3)
   90       CONTINUE
            IF (K.NE.0)
     >        CALL XABORT( 'XELDCL: DATA ERROR')
         ENDIF
         IF( LLSYM )THEN
C
C           REPLACE "KEYTYP" AND UNFOLD FOR EXTERNAL SYMMETRIES
            DO 320 IZ=MINGRI(3),1,-1
            IOF1=(IZ-1)*MINGRI(1)*MINGRI(2)
            IOF2=(IZ+MEDGRI(3)-1)*MAXGRI(1)*MAXGRI(2)
            DO 310 IY=MINGRI(2),1,-1
            DO 310 IX=MINGRI(1),1,-1
            IF(IOF2+(IY+MEDGRI(2)-1)*MAXGRI(1)+IX+MEDGRI(1).NE.
     >         IOF1+(IY-1)*MINGRI(1)+IX )THEN
       IF(KEYTYP(IOF2+(IY+MEDGRI(2)-1)*MAXGRI(1)+IX+MEDGRI(1)).NE.0)THEN
            CALL XABORT( 'XELDCL: PROBLEMS TO UNFOLD')
       ELSE
            KEYTYP(IOF2+(IY+MEDGRI(2)-1)*MAXGRI(1)+IX+MEDGRI(1))=
     >                    KEYTYP(IOF1+(IY-1)*MINGRI(1)+IX)
            KEYTYP(IOF1+(IY-1)*MINGRI(1)+IX)= 0
            ITGEOM(IOF2+(IY+MEDGRI(2)-1)*MAXGRI(1)+IX+MEDGRI(1))=
     >                    ITGEOM(IOF1+(IY-1)*MINGRI(1)+IX)
            ITGEOM(IOF1+(IY-1)*MINGRI(1)+IX)= 0
       ENDIF
            ENDIF
  310       CONTINUE
  320       CONTINUE
            IF( LCLSYM(3).NE.0)THEN
               DO 330 IZ=1,MINGRI(3)
               DO 330 IY=1,MAXGRI(2)
               DO 330 IX=1,MAXGRI(1)
               IOF1=((IZ-1)*MAXGRI(2)+(IY-1))*MAXGRI(1)+IX
               IOF2=((MAXGRI(3)-IZ)*MAXGRI(2)+(IY-1))*MAXGRI(1)+IX
               IF( KEYTYP(IOF1).NE.KEYTYP(IOF2) )THEN
                  IF( KEYTYP(IOF1).GT.KEYTYP(IOF2) )THEN
                     KEYTYP(IOF2)=     KEYTYP(IOF1)
                     ITGEOM(IOF2)= MOD(MAXTUR+ITGEOM(IOF1),2*MAXTUR)
                  ELSE
                     KEYTYP(IOF1)=     KEYTYP(IOF2)
                     ITGEOM(IOF1)= MOD(MAXTUR+ITGEOM(IOF2),2*MAXTUR)
                  ENDIF
               ENDIF
C----GM
      IF(IZ .EQ. (MAXGRI(3)+1-IZ)) THEN
        IKG=KEYTYP(IOF1)
        IKT(1)=ITGEOM(IOF1)
        IKT(2)=MOD(MAXTUR+IKT(1),2*MAXTUR)
        IF(IKG .GT. 0) THEN 
          WRITE(GEOCV,'(3A4)')
     >    CELLT(3*IKG-2),CELLT(3*IKG-1),CELLT(3*IKG)       
          IF(IPRT .GT. 10)        
     >      WRITE(IOUT,8000) 'Z-Z',GEOCV,
     >      CTURN(IKT(1)),CTURN(IKT(2))
          IF(.NOT. LELCSY(IPGEOM,GEOCV,IKT) ) THEN 
            WRITE(IOUT,8001) 'Z-Z',GEOCV,
     >      CTURN(IKT(1)),CTURN(IKT(2))
            CALL XABORT('XELDCL: INVALID SYMMETRY FOR CELL')
          ENDIF
        ENDIF
      ENDIF
C----GM
  330          CONTINUE
            ENDIF
            IF( LCLSYM(2).NE.0)THEN
               DO 340 IZ=1,MAXGRI(3)
               DO 340 IY=1,MINGRI(2)
               DO 340 IX=1,MAXGRI(1)
               IOF1=((IZ-1)*MAXGRI(2)+(IY-1))*MAXGRI(1)+IX
               IOF2=((IZ-1)*MAXGRI(2)+(MAXGRI(2)-IY))*MAXGRI(1)+IX
               IF( KEYTYP(IOF1).NE.KEYTYP(IOF2) )THEN
                  IF( KEYTYP(IOF1).GT.KEYTYP(IOF2) )THEN
                     KEYTYP(IOF2)=       KEYTYP(IOF1)
                     ITGEOM(IOF2)= ITURN(ITGEOM(IOF1),2)
                  ELSE
                     KEYTYP(IOF1)=       KEYTYP(IOF2)
                     ITGEOM(IOF1)= ITURN(ITGEOM(IOF2),2)
                  ENDIF
               ENDIF
C----GM
      IF(IY .EQ. (MAXGRI(2)+1-IY) ) THEN
        IKG=KEYTYP(IOF1)
        IKT(1)=ITGEOM(IOF1)
        IKT(2)=ITURN(IKT(1),2)
        IF(IKG .GT. 0) THEN 
          WRITE(GEOCV,'(3A4)')
     >    CELLT(3*IKG-2),CELLT(3*IKG-1),CELLT(3*IKG)       
          IF(IPRT .GT. 10)        
     >      WRITE(IOUT,8000) 'Y-Y',GEOCV,
     >      CTURN(IKT(1)),CTURN(IKT(2))
          IF(.NOT. LELCSY(IPGEOM,GEOCV,IKT) ) THEN 
            WRITE(IOUT,8001) 'Y-Y',GEOCV,
     >      CTURN(IKT(1)),CTURN(IKT(2))
            CALL XABORT('XELDCL: INVALID SYMMETRY FOR CELL')
          ENDIF
        ENDIF
      ENDIF
C----GM
  340          CONTINUE
            ENDIF
            IF( LCLSYM(1).NE.0)THEN
               DO 350 IZ=1,MAXGRI(3)
               DO 350 IY=1,MAXGRI(2)
               DO 350 IX=1,MINGRI(1)
               IOF1=((IZ-1)*MAXGRI(2)+(IY-1))*MAXGRI(1)+IX
               IOF2=((IZ-1)*MAXGRI(2)+(IY-1))*MAXGRI(1)+MAXGRI(1)+1-IX
               IF( KEYTYP(IOF1).NE.KEYTYP(IOF2) )THEN
                  IF( KEYTYP(IOF1).GT.KEYTYP(IOF2) )THEN
                     KEYTYP(IOF2)=       KEYTYP(IOF1)
                     ITGEOM(IOF2)= ITURN(ITGEOM(IOF1),1)
                  ELSE
                     KEYTYP(IOF1)=       KEYTYP(IOF2)
                     ITGEOM(IOF1)= ITURN(ITGEOM(IOF2),1)
                  ENDIF
               ENDIF
C----GM
      IF(IX .EQ. (MAXGRI(1)+1-IX)) THEN
        IKG=KEYTYP(IOF1)
        IKT(1)=ITGEOM(IOF1)
        IKT(2)=ITURN(IKT(1),1)
        IF(IKG .GT. 0) THEN 
          WRITE(GEOCV,'(3A4)')
     >    CELLT(3*IKG-2),CELLT(3*IKG-1),CELLT(3*IKG)       
          IF(IPRT .GT. 10)        
     >      WRITE(IOUT,8000) 'X-X',GEOCV,
     >      CTURN(IKT(1)),CTURN(IKT(2))
          IF(.NOT. LELCSY(IPGEOM,GEOCV,IKT) ) THEN 
            WRITE(IOUT,8001) 'X-X',GEOCV,
     >      CTURN(IKT(1)),CTURN(IKT(2))
            CALL XABORT('XELDCL: INVALID SYMMETRY FOR CELL')
          ENDIF
        ENDIF
      ENDIF
C----GM
  350          CONTINUE
            ENDIF
         ENDIF
C
C        FIND ALL DIFFERENT GEOMETRIES
C         CALL LCMGET(IPGEOM,'CELL',CELLT)
         IF( IPRT.GT.1 )THEN
            WRITE(IOUT,'(1H )')
            NXC= 1
            WRITE(IOUT,'(25H ===> CELL TYPES ARE:    /)')
            DO 410 IP= 1, (9+ISUB2)/10
               NXM= MIN( ISUB2, NXC+9 )
               WRITE(IOUT,'(1H ,10(I4.4,8X))')
     >         (IB1,IB1=NXC,NXM)
               WRITE(IOUT,'(1H ,30A4)')
     >         (CELLT(3*IB1-2),CELLT(3*IB1-1),CELLT(3*IB1),IB1=NXC,NXM)
               NXC= NXC + 10
  410       CONTINUE
C
C           PRINTING ASSEMBLY MAP
            CPLAN= BLANC
            NLINP= 3+(MAXGRI(2)+1)*((9+MAXGRI(1))/10+1)
            DO 430 IZ=1,MAXGRI(3)
            WRITE(IOUT,'(1H )')
            IF(NDIM.EQ.3)THEN
               WRITE(CPLAN,'(4H (Z=,I3,1H))') IZ
            ENDIF
            WRITE(IOUT,'(/32H UNFOLD TYPE CELL MAP FOR PLANE ,A8)')CPLAN
            NXC= 1
            DO 440 IP  = 1, (9 + MAXGRI(1)) / 10
               NXM= MIN( MAXGRI(1), NXC+9 )
               WRITE(IOUT,'(1X,A8,1X,10(A4,I3,A4))')
     >            CPLAN, (' X= ',IR,' ROT',IR=NXC,NXM)
               NXC = NXC + 10
  440       CONTINUE
            WRITE(IOUT,'(1H )')
            DO 430 IY=1,MAXGRI(2)
            IOFF=((IZ-1)*MAXGRI(2)+(IY-1))*MAXGRI(1)
            NXC= 1
            DO 450 IP  = 1, (9 + MAXGRI(1)) / 10
               NXM= MIN( MAXGRI(1), NXC+9 )
               WRITE(IOUT,'(1X,A4,I3,2H=>,10(I7,1X,A2,1X))')
     >             ' Y= ',IY,(KEYTYP(IOFF+IR),
     >                        CTURN(ITGEOM(IOFF+IR)),IR=NXC,NXM)
               NXC = NXC + 10
  450       CONTINUE
            WRITE(IOUT,'(1H )')
  430       CONTINUE
         ENDIF
         NGEOME= 0
         DO 50 IB1= 1, NBLOCK
            IC1= KEYTYP(IB1)
            IT1= ITGEOM(IB1)
            IF(IC1.LE.0.OR.IC1.GT.ISUB2 )THEN
               CALL XABORT('XELDCL: INVALID TYPE #')
            ENDIF
            NGEOME= NGEOME + 1
            CELLG(3*NGEOME-2)= CELLT(3*IC1-2)
            CELLG(3*NGEOME-1)= CELLT(3*IC1-1)
            CELLG(3*NGEOME  )= CELLT(3*IC1  )
            KEYTRN(NGEOME)= IT1
            WRITE( GEOC1(1: 4),'(A4)') CELLT(3*IC1-2)
            WRITE( GEOC1(5: 8),'(A4)') CELLT(3*IC1-1)
            WRITE( GEOC1(9:12),'(A4)') CELLT(3*IC1  )
C           SEARCH FOR SIMILAR GEOMETRIES IN PREVIOUS ONES
            IF( IB1.NE.1 )THEN
               DO 40 IB2= 1, IB1-1
                  IC2= KEYTYP(IB2)
                  IT2= ITGEOM(IB2)
                  IF( IT1.NE.IT2 ) GO TO 40
                  WRITE( GEOC2(1: 4),'(A4)') CELLT( 3*IC2-2 )
                  WRITE( GEOC2(5: 8),'(A4)') CELLT( 3*IC2-1 )
                  WRITE( GEOC2(9:12),'(A4)') CELLT( 3*IC2   )
                  IF( GEOC1.EQ.GEOC2 )THEN
                     KEYGEO(IB1)= KEYGEO(IB2)
                     NGEOME= NGEOME-1
                     GO TO 50
                  ENDIF
   40          CONTINUE
            ENDIF
C           ANALYSE NEW GEOMETRY
            CALL LCMSIX(IPGEOM,GEOC1,1)
            CALL XELPRC(IPGEOM,GEOC1,NDIM,NNCYL,NNSUR,NNVOL,MAXREM,
     >      ISPLT)
            IF( NNVOL.NE.0 )THEN
               NSURO(NGEOME)= -NNSUR
               NVOLO(NGEOME)=  NNVOL
               IDLDIM(NGEOME)= NTOTCO
               NTOTCO= NTOTCO + NNCYL + 3
               MAXRO= MAXRO + MAXREM
               IGEO= NGEOME
            ELSE
               NGEOME= NGEOME-1
               IGEO= -1
            ENDIF
            KEYGEO(IB1)= IGEO
            CALL LCMSIX(IPGEOM,' ',2)
   50    CONTINUE
         IF( IPRT.GT.1 )THEN
            WRITE(IOUT,'(1H )')
            NXC= 1
            WRITE(IOUT,'(25H ===> PHYSICAL CELLS ARE:    /)')
            DO 411 IP= 1, (9+NGEOME)/10
               NXM= MIN( NGEOME, NXC+9 )
               WRITE(IOUT,'(1H ,10(I4.4,8X))')
     >         (IB1,IB1=NXC,NXM)
               WRITE(IOUT,'(1H ,30A4)') (CELLG(3*IB1-2),
     >         CELLG(3*IB1-1),CELLG(3*IB1),IB1=NXC,NXM)
               WRITE(IOUT,'(1H ,10(A7,5X))')
     >         ('TURN '//CTURN(KEYTRN(IB1)),IB1=NXC,NXM)
               NXC= NXC + 10
  411       CONTINUE
         ENDIF
C
C        RESTORE *KEYTYP* AND *KEYGEO* VALUES
         NTYP2= NGEOME
         DO 56 IB1= 1, NBLOCK
            KEYTYP(IB1)= KEYGEO(IB1)
   56    CONTINUE
         DO 57 IC1= 1, NTYP2
            KEYGEO(IC1)= IC1
   57    CONTINUE
C
C        DELETE ALL VIRTUAL CELLS
         NTYP= 0
         DO 150 ITYP= 1, NTYP2
            IGEO= KEYGEO(NTYP+1)
            IF( IGEO.EQ.-1 )THEN
               DO 100 IBLK= 1, NBLOCK
                  IOLTYP= KEYTYP(IBLK)
                  IF( IOLTYP.EQ.NTYP+1 )THEN
                     KEYTYP(IBLK)= 0
                  ELSEIF( IOLTYP.GT.NTYP+1 )THEN
                     KEYTYP(IBLK)= KEYTYP(IBLK)-1
                  ENDIF
  100          CONTINUE
               DO 110 JTYP= NTYP+2, NTYP2
                  CELLG(3*JTYP-5)= CELLG(3*JTYP-2)
                  CELLG(3*JTYP-4)= CELLG(3*JTYP-1)
                  CELLG(3*JTYP-3)= CELLG(3*JTYP  )
                  KEYTRN(JTYP-1)= KEYTRN(JTYP)
                  KEYGEO(JTYP-1)= KEYGEO(JTYP)
  110          CONTINUE
            ELSE
               NTYP= NTYP+1
            ENDIF
  150    CONTINUE
      ELSE
C
C        NO CELL IN THE GEOMETRY
         IF( NTYPO.NE.1 )
     >      CALL XABORT('XELDCL: INVALID GEOMETRY TYPE '//GEONAM)
         NGEOME=    1
         NTYP= NTYPO
         READ(GEONAM( 1: 4),'(A4)') CELLG(3*NGEOME-2)
         READ(GEONAM( 5: 8),'(A4)') CELLG(3*NGEOME-1)
         READ(GEONAM( 9:12),'(A4)') CELLG(3*NGEOME  )
         KEYGEO(1)= 1
         KEYTYP(1)= 1
         MRGCEL(1)= 1
         ITGEOM(1)= 1
         KEYTRN(1)= 1
C
         CALL XELPRC(IPGEOM,GEONAM,NDIM,NNCYL,NNSUR,NNVOL,MAXRO,ISPLT)
         IF( NNVOL.EQ.0 )
     >      CALL XABORT('XELDCL: ONE-CELL VIRTUAL GEOMETRY '//GEONAM)
         NSURO(1)= -NNSUR
         NVOLO(1)=  NNVOL
         IDLDIM(1)= NTOTCO
         NTOTCO=    NTOTCO + NNCYL + 3
      ENDIF
      IF( IPRT.GT.1 )THEN
         WRITE(IOUT,'(/35H ONE TRACKING FILE TO BE ATTACHED    /'//
     >              '1H ,12X,14H UNDER NAME : ,A12 )') GEONAM
      ENDIF
C
C     DEFINITION OF INDEX VALUES, TO LOOK AT THE DOMAIN
C     EVERYTHING BELOW WAS FINE..., SO WHY NOT KEEP IT ?
       NGIDL= 0
      DO 210 IGEO= 1, NGEOME
          IF( NSURO(IGEO).GE.0 )
     >       CALL XABORT('XELDCL: GEOMETRY NOT FOUND')
          IF( NVOLO(IGEO).LE.0 )
     >       CALL XABORT('XELDCL: GEOMETRY NOT FOUND')
          IDLGEO(IGEO)=  NGIDL - NSURO(IGEO) + 1
           NGIDL= NVOLO(IGEO) + IDLGEO(IGEO)
  210 CONTINUE
      NTIDL = 0
      NPROB = 0
      DO 220 ITYP= 1, NTYP
            IGEO= KEYGEO(ITYP)
          IF( IGEO.LE.0 )
     >       CALL XABORT('XELDCL: BLOC NOT FOUND')
          IDLTYP(ITYP)= NTIDL - NSURO(IGEO) + 1
          IDLPRB= NPROB  + (1-NSURO(IGEO))*(2-NSURO(IGEO))/2
          NTIDL = NVOLO(IGEO) + IDLTYP(ITYP)
          NPROB = NVOLO(IGEO)*(NVOLO(IGEO)-2*NSURO(IGEO)+3)/2+IDLPRB
  220 CONTINUE
      NUNKO= 0
      DO 230 IBLK= 1, NBLOCK
            ITYP= KEYTYP(IBLK)
          IF( ITYP.LT.0 )
     >       CALL XABORT('XELDCL: CELL NOT FOUND')
          IF( ITYP.EQ.0 )GO TO 230
            IGEO= KEYGEO(ITYP)
          IDLBLK(IBLK)= NUNKO - NSURO(IGEO) + 1
          NUNKO= NVOLO(IGEO) + IDLBLK(IBLK)
  230 CONTINUE
C
      IF( IPRT.GT.10 )THEN
C
C     PRINTING INDEX VECTORS
          WRITE(IOUT,'(1H )')
          WRITE(IOUT,'(1H0,A6,4H -->,I6)') 'MAXRO', MAXRO
          WRITE(IOUT,'(1H0,A6,4H -->,I6)') 'NTOTCO', NTOTCO
          WRITE(IOUT,'(1H0,A6,4H -->,3(I6,1X))') 'LCLSYM', LCLSYM
          WRITE(IOUT,'(1H0,A6,4H    ,5(A6,2X))') 'GEOM #',
     >    ' NSURO', ' NVOLO', 'IDLGEO', 'IDLDIM', 'KEYTRN'
          DO 250 IGEO= 1, NGEOME
              WRITE(IOUT,'(1H ,I6,4H -->,5(I6,2X))') IGEO,
     >   NSURO(IGEO),NVOLO(IGEO),IDLGEO(IGEO),IDLDIM(IGEO),KEYTRN(IGEO)
  250     CONTINUE
C
          WRITE(IOUT,'(1H )')
          WRITE(IOUT,'(1H0,A6,4H    ,2(A6,2X))') 'BLOC #',
     >    'KEYGEO', 'IDLTYP'
          DO 260 ITYP= 1, NTYP
              WRITE(IOUT,'(1H ,I6,4H -->,2(I6,2X))') ITYP,
     >        KEYGEO(ITYP), IDLTYP(ITYP)
  260     CONTINUE
C
          WRITE(IOUT,'(1H )')
          WRITE(IOUT,'(1H0,A6,4H    ,3(A6,2X))') 'CELL #',
     >    'KEYTYP', 'ITGEOM',  'IDLBLK'
          DO 270 IBLK= 1, NBLOCK
              WRITE(IOUT,'(1H ,I6,4H -->,3(I6,2X))') IBLK,
     >        KEYTYP(IBLK), ITGEOM(IBLK), IDLBLK(IBLK)
  270     CONTINUE
      ENDIF
C
      RETURN 
C----
C  FORMAT
C----
 8000 FORMAT(' XELDCL NOW TESTING SYMMETRY ',A3,' FOR ',
     > A12,1X,'WITH ROTATION',1X,A2,' AND ',A2)
 8001 FORMAT('            INVALID SYMMETRY ',A3,' FOR ',
     > A12,1X,'WITH ROTATION',1X,A2,' AND ',A2)
      END
